# -*- coding: utf-8 -*-
## export_Wirkungsdiagramm.py
## Gui zum Export eines Wirkungsdiagramms als Simulationsmodell
## © Claus Albowski

## Importe:
import platform
import os
versionsNummer=platform.python_version()[0]
if versionsNummer=='2':
    from Tkinter import *
    import tkFileDialog as fileDialog
else:
    from tkinter import *
    import tkinter.filedialog as fileDialog

from wirkungsModell import *
from wirkungsFensterElemente import *
#### Dialoge
from dialoge import * # eigene
#### Spezialfunktionen
from random import randint
from wdController import WDController


class ExportApplication(Frame):
    
    def __init__(self, master=None, zeichenflaechenBreite=800, zeichenflaechenHoehe=800):
        Frame.__init__(self, master)
        self.breite = zeichenflaechenBreite
        self.hoehe = zeichenflaechenHoehe
        self.__cont=WDController.GibWDController(self)

        top = self.winfo_toplevel()
        ## Menübereich
        self.menuBar = Menu(top)
        top['menu'] = self.menuBar
        #### Datei-Menu
        self.dateiSubMenu = Menu(self.menuBar)
        self.menuBar.add_cascade(label='Datei', font='Sans 10 bold', menu=self.dateiSubMenu)
        self.dateiSubMenu.add_command(label='Daten einlesen', font='Sans 10 bold', command=self.onLesen)
        self.dateiSubMenu.add_command(label='Daten exportieren', font='Sans 10 bold', command=self.onSpeichern)
        self.dateiSubMenu.add_command(label='Beenden', font='Sans 10 bold', command=self.onBeenden)
        #### Modellentwicklungs-Menu
        self.neuSubMenu = Menu(self.menuBar)
        self.menuBar.add_cascade(label='Neu', font='Sans 10 bold', menu=self.neuSubMenu)
##        self.neuSubMenu.add_command(label='Neues Modell', font='Sans 10 bold', command=self.onNeuesModell)
        self.neuSubMenu.add_command(label='neuer Faktor', font='Sans 10 bold', command=self.onFaktorNeu)
        self.neuSubMenu.add_command(label='neue Wirkung', font='Sans 10 bold', command=self.onWirkungNeu)
        self.neuSubMenu.add_command(label='neue Wirkung mit Maus', font='Sans 10 bold', command=self.onWirkungMaus)
        #### Bearbeiten-Menu
        self.bearbeitenSubMenu = Menu(self.menuBar)
        self.menuBar.add_cascade(label='Bearbeiten', font='Sans 10 bold', menu=self.bearbeitenSubMenu)
        self.bearbeitenSubMenu.add_command(label='Faktor ist Bestandsfaktor', font='Sans 10 bold', command=self.onFaktorBestand)
        self.bearbeitenSubMenu.add_command(label='Faktor ist Fluss', font='Sans 10 bold', command=self.onFaktorFluss)
        self.bearbeitenSubMenu.add_command(label='Faktor ist Parameter', font='Sans 10 bold', command=self.onFaktorParameter)
        self.bearbeitenSubMenu.add_command(label='Faktor ist Zwischengroesse', font='Sans 10 bold', command=self.onFaktorZwischengroesse)
        self.bearbeitenSubMenu.add_command(label='Faktornamen aendern', font='Sans 10 bold', command=self.onFaktorBearbeiten)
        self.bearbeitenSubMenu.add_command(label='Wirkung bearbeiten', font='Sans 10 bold', command=self.onWirkungBearbeiten)
        #### Loeschen-Menu
        self.loeschenSubMenu = Menu(self.menuBar)
        self.menuBar.add_cascade(label=u'Löschen', font='Sans 10 bold', menu=self.loeschenSubMenu)
        self.loeschenSubMenu.add_command(label=u'Faktor löschen', font='Sans 10 bold', command=self.onLoescheFaktor)
        self.loeschenSubMenu.add_command(label=u'Wirkung löschen', font='Sans 10 bold', command=self.onLoescheWirkung)
        #### Darstellungs-Menu
        self.darstellungSubMenu = Menu(self.menuBar)
        self.menuBar.add_cascade(label='Darstellung', font='Sans 10 bold', menu=self.darstellungSubMenu)
        self.darstellungSubMenu.add_command(label=u'Fenstergröße ändern', font='Sans 10 bold', command=self.onFenster)
        self.darstellungSubMenu.add_command(label='Modelldaten zeigen', font='Sans 10 bold', command=self.onModelldatenZeigen)
        #### Hilfe-Menu
        self.hilfeSubMenu = Menu(self.menuBar)
        self.menuBar.add_cascade(label='Hilfe', font='Sans 10 bold', menu=self.hilfeSubMenu)
        self.hilfeSubMenu.add_command(label='Beispiel zeigen', font='Sans 10 bold', command=self.zeichne_Objekte)
        self.hilfeSubMenu.add_command(label='Hilfe zum Export', font='Sans 10 bold', command=self.ZeigeHilfeText1)
        self.hilfeSubMenu.add_command(label='Hilfe zu den Dateien', font='Sans 10 bold', command=self.ZeigeHilfeText2)
        self.hilfeSubMenu.add_command(label='Hilfe zu Kurztasten', font='Sans 10 bold', command=self.ZeigeHilfeText4)
        self.hilfeSubMenu.add_command(label=u'Hilfe zum Ändern der Modellfenstergröße', font='Sans 10 bold', command=self.ZeigeHilfeText5)

        ## Anlegen der Attribute        
        self.__w=WirkungsModell.GibWirkungsModell(self)
        self.__click_position = (100,100)
        self.__click_ID = None     # fuer linken Mausklick
        self.__neueWirkung = False # fuer grafische Verknuepfung
        self.__ausgewaehlt = None  # fuer Ziehen mit linker Maustaste gedrueckt
        self.__release_IDs = None   # fuer linke Maustaste loslassen
        ## Tk-Variable [Daten für Widgets]
        self.__modellVariable = StringVar()
        self.__modellVariable.set('')
        self.__geaendert = False
        self.__HilfeMausAnzeigen = True
        self.__aktuellesVerzeichnis = '.'

        ## Definition von Mausereignissen fuer das Fenster
        self.winfo_toplevel().bind("<Control-o>", self.onControl_o)
        self.winfo_toplevel().bind("<Control-s>", self.onControl_s)
        self.winfo_toplevel().bind("<Control-p>", self.onControl_p)
        self.winfo_toplevel().bind("<Alt-n>", self.onAlt_n)
        self.winfo_toplevel().bind("<Alt-f>", self.onAlt_f)
        self.winfo_toplevel().bind("<Alt-w>", self.onAlt_w)
        self.winfo_toplevel().bind("<Alt-m>", self.onAlt_m)
        self.winfo_toplevel().bind("<Alt-z>", self.onAlt_z)

        ## Oberflaeche erzeugen
        self.createWidgets()
        self.pack()

    def onControl_o(self,evt): self.onLesen()
    def onControl_s(self,evt): self.onSpeichern()
    def onControl_p(self,evt): self.onBild()
    def onAlt_n(self,evt): self.onNeuesModell()
    def onAlt_f(self,evt): self.onFaktorNeu()
    def onAlt_w(self,evt): self.onWirkungNeu()
    def onAlt_m(self,evt): self.onWirkungMaus()
    def onAlt_z(self,evt): self.onModelldatenZeigen()


    def createWidgets(self):
        '''Einsatz des grid - Geometriemanagers'''
        # 1. Zeile
        self.modell_Label = Label(self, font='Sans 12', text=" Modellname: ")
        self.modell_Label.grid(row=0,column=0)
        self.modellNameLabel = Label(self, textvariable=self.__modellVariable, width=22, font='Sans 14 bold italic', bg='white', fg='black')
        self.modellNameLabel.grid(row=0,column=1)
        self.beenden_Button = Button(self, text="Beenden", font='Sans 10 bold', command=self.onBeenden)
        self.beenden_Button.grid(row=0,column=2)
        # 2. Zeile
        self.modellFrame=Frame(self)
        self.modellFrame.grid(row=1,column=0, columnspan=3)
        self.zeichenflaeche = Canvas(self.modellFrame,
                                     width=self.breite,
                                     height=self.hoehe,
                                     bg='white',
                                     scrollregion=(0, 0, 1800, 1200))
        zf=self.zeichenflaeche # lokale Hilfsvariable
        zf.grid(row=0,column=0)
        self.scrollX=Scrollbar(self.modellFrame,orient=HORIZONTAL,command=zf.xview)
        self.scrollY=Scrollbar(self.modellFrame,orient=VERTICAL,command=zf.yview)
        self.scrollX.grid(row=1,column=0, sticky=W+E)
        self.scrollY.grid(row=0,column=1, sticky=N+S)
        zf['xscrollcommand'] = self.scrollX.set
        zf['yscrollcommand'] = self.scrollY.set

        # Binden von Ereignissen an die Zeichenflaeche:
        zf.bind("<Button-1>", self.links_Mausklick)
        zf.bind("<Button-3>", self.rechts_Mausklick)
        zf.bind("<ButtonRelease>", self.links_loslassen)
        zf.bind("<Motion>", self.bewegt)


    #### --------- Dateimenue ------------------------------
    def onLesen(self):
        '''Der Modellname wird zwingend aus dem Dateinamen entnommen.'''
        if not self.__cont.IstLeer():
            dialog=BestaetigungsDialog(self, u'Aktuelles Modell löschen?', 'Wollen Sie das wirklich?')
            if dialog.result==None: return
            if dialog.result==False: return
            if dialog.result==True:
                self.__w.LoescheModell()
                self.__cont.LoescheAlle(self.zeichenflaeche)
                self.__modellVariable.set('')
        verzeichnis = 'WirkungsModelle'
        if self.__aktuellesVerzeichnis!='.': verzeichnis = self.__aktuellesVerzeichnis
        dateiname=fileDialog.askopenfilename(parent=self,
                                                     defaultextension='wirkdia',
                                                     initialdir=verzeichnis,
                                                     title="Modelldaten laden")
        if (dateiname==None) or dateiname==():
            MeldungDialog(self,'Info', 'Keine Daten gelesen!')
            return
        else:
            #Modellwechsel verhindern
            modellName=self.__w.ModellNameAusDateiNamen(dateiname)
            if not(self.SetzeModellNamen(modellName)): return
            daten=self.__w.Lies(dateiname)
            if daten==None: return
            # extrahiert die Werte und erzeugt die Faktoren und Wirkungen
            zf=self.zeichenflaeche
            self.__geaendert = False
            for element in daten:
                if element[0]=='Faktor':
                    name=element[1]
                    try: pos=element[2]
                    except IndexError:
                        pos=(randint(100,200),randint(100,200))
                    fe=FaktorElement(zf,name,pos)
                    faktor=Faktor.GibFaktorZuNamen(name)
                    self.__cont.NeuerFaktor(name,faktor,fe)
                elif element[0]=='Wirkung':
                    if len(element)>4:
                        von,auf,art,bogenOr=element[1:]
                    else:
                        von,auf,art=element[1:]
                        bogenOr=1
                    wi=Wirkung.HatWirkung(von, auf)
                    if wi: # existiert
                        we=WirkungElement(zf, von, auf, art)
                        name=we.GibTag()
                        self.__cont.NeueWirkung(name,wi,we)
                        if bogenOr==-1:
                            self.__cont.AendereBogenOrientierung(zf,name)                    
        
    def onSpeichern(self):
        '''Es wird zunaechst geprueft, ob ein Modellname existiert.'''
        if self.__modellVariable.get()=='':
            self.SetzeModellNamen()
        verzeichnis = 'SimulationsModelle'
        if self.__aktuellesVerzeichnis!='.': verzeichnis = self.__aktuellesVerzeichnis
        dateiname=self.__modellVariable.get()+'.simdia'
        dateiname=fileDialog.asksaveasfilename(parent=self,
                                                       initialfile=dateiname,
                                                       defaultextension='simdia',
                                                       initialdir=verzeichnis,
                                                       title="Export in Simulationsdatei")
        if dateiname!=None:
            self.__w.SichereModellDaten()
            daten=self.PositionenEinfuegen(self.__w.GibModellDaten())
            self.__w.Speichere(dateiname,daten)
            self.__geaendert = False
        else:
            MeldungDialog('Info', 'Keine Daten geschrieben!')


    def PositionenEinfuegen(self, alleDaten):
        zf=self.zeichenflaeche
        datenMitPositionen=[]
        for daten in alleDaten:
            if daten[0] in ['Bestand','Fluss','Parameter','Zwischengroesse']:
                x,y,xe,ye=zf.coords(daten[1])
                if daten[0] =='Fluss':
                    datenMitPositionen.append((daten[0],
                                               daten[1],
                                               daten[2],
                                               daten[3],
                                               (x,y)))
                elif daten[0] in ['Bestand','Parameter']:
                    datenMitPositionen.append((daten[0],daten[1],daten[2],(x,y)))
                else:
                    datenMitPositionen.append((daten[0],daten[1],(x,y)))
            else:
                datenMitPositionen.append(daten)
        return datenMitPositionen
        
    def SetzeModellNamen(self, modellName=''):
        if modellName=='': # aus dem Menue aufgerufen
            dialog=EingabeDialog(self, u'Modellnamen wählen', 'Namen eingeben')
            if dialog.result==None: return # cancel ausgewaehlt
            modellName=dialog.result
        if self.__modellVariable.get()!='':
            if self.__modellVariable.get()!=modellName:
                MeldungDialog(self,'Warnung','Zum Modellwechsel \n das Programm \n neu starten!')
                return False
        self.__modellVariable.set(modellName)
        return True

    def onHome(self):
        self.__aktuellesVerzeichnis = os.environ["HOME"]
        os.chdir(os.environ["HOME"])

    def onVerzeichnis(self):
        verzeichnisName=self.__aktuellesVerzeichnis
        if verzeichnisName=='.': verzeichnisName=''
        dialog=EingabeDialog(self, u'aktuelles Verzeichnis neu wählen', 'Namen eingeben',verzeichnisName)
        if dialog.result==None: return # cancel ausgewaehlt
        verzeichnisName=dialog.result
        try:
            os.chdir(verzeichnisName)
        except OSError as e:
            MeldungDialog(self,'Fehler','Verzeichnis existiert nicht!')
            return
        self.__aktuellesVerzeichnis = verzeichnisName

    def onBeenden(self):
        if self.__geaendert:
            dialog=BestaetigungsDialog(self, 'Achtung!', 'Aktuelles Modell sichern?')
            if dialog.result==None: return
            if dialog.result==True:
                self.onSpeichern()
                return
        self.quit()
        
    #### ---------- Modellentwicklungs-Menue -------------------------------------    
    def onNeuesModell(self):
        '''Strategie:
        - noch kein Name und kein Modell: -> Namen festlegen
        - vorhandenes Modell mit neuem Namen: -> Abfrage, ob gewollt'''
        if self.__cont.IstLeer():
            self.SetzeModellNamen()
            return
        dialog=BestaetigungsDialog(self, u'Aktuelles Modell löschen?', 'Wollen Sie das wirklich?')
        if dialog.result==None:
            MeldungDialog(self,'Ergebnis :','abgebrochen')
            return
        if dialog.result==True:
            self.__w.LoescheModell()
            self.__cont.LoescheAlle(self.zeichenflaeche)
            self.__modellVariable.set('')
        dialog=EingabeDialog(self, u'Neuer Name für das Modell', 'Namen eingeben')
        name=dialog.result
        if dialog.result==None: return # cancel ausgewaehlt
        self.__modellVariable.set(name)
        self.__geaendert = True
    
    def onFaktorNeu(self):
        dialog=EingabeDialog(self, 'neuer Faktor', 'Namen eingeben')
        name=dialog.result
        if dialog.result==None: return # cancel ausgewaehlt
        if dialog.result=='': return # keine Eingabe
        faktor=self.__w.NeuerFaktor(name)
        if faktor!=None:
            fe=FaktorElement(self.zeichenflaeche,name)
            self.__cont.NeuerFaktor(name,faktor,fe)
            
    def onWirkungNeu(self):
        vonDialog=AuswahlDialog(self,u'Ausgangsfaktor wählen',self.__w.GibAlleFaktorNamen(),True)
        if vonDialog.result==None: return # bei cancel None
        von=vonDialog.result
        ziele=[]+self.__w.GibAlleFaktorNamen() # echte Kopie
        if  von in ziele: ziele.remove(von) # Keine Wirkung auf sich selbst
        aufDialog=AuswahlDialog(self,
                                u'Zielfaktor wählen',
                                ziele,
                                True)
        if aufDialog.result==None: return # bei cancel None
        auf=aufDialog.result
        wi=self.__w.NeueWirkung(von, auf,'+')
        we=WirkungElement(self.zeichenflaeche, von, auf)
        self.__cont.NeueWirkung(we.GibTag(),wi,we)
        self.__geaendert = True

    def onWirkungMaus(self):
        '''Zustand wird durch self.__neueWirkung gesetzt'''
        if self.__HilfeMausAnzeigen:
            dialog=MeldungDialog(self,'Wirkung mit Maus',
                                 'Ausgangsfaktor anklicken,\n'
                                 +'Maus zu Zielfaktor ziehen;\n'
                                 +'jeweils linke obere Ecke!')
        self.__HilfeMausAnzeigen = False
        self.__neueWirkung = True
        

    #### ----------- Bearbeiten-Menu ----------------------------------------
    def onFaktorBearbeiten(self, alterName=None):
        '''die hier einzig moegliche Bearbeitung ist das Aendern des Namens'''
        if alterName==None:
            alterName=self.FaktorWaehlenDialog()
            if alterName==None: return # cancel an geklickt
        dialog=EingabeDialog(self,'Aendern des Faktornamen','bisher: '+alterName)
        neuerName=dialog.result
        if neuerName==None: return # cancel an geklickt
        if not(self.__w.NeuerNameOK(neuerName)):
            self.Meldung('Name unzulaessig')
            return
        if alterName!=neuerName: # sollte wegen der Pruefung ok sein
            self.__w.AendereFaktorNamen(alterName,neuerName)
            self.__cont.AendereFaktor(self.zeichenflaeche,alterName,neuerName)
            self.__cont.AendereWirkungenZu(self.zeichenflaeche,alterName,neuerName)
            self.__geaendert = True

    def onFaktorBestand(self):
        '''weist einem Faktor die Eigenschaft Bestand zu sein zu.
        Zu diesem Faktor werden alle Wirkungen auf ihn geloescht.'''
        ## solange die Darstellung nicht geaendert wird, muss die Aktion
        ## nicht ueber den Controller gehen
        dialog=AuswahlDialog(self,
                             'Faktor waehlen',
                             self.__w.GibAlleFaktorNamen(),
                             True)
        if dialog.result==None: return # bei cancel None
        faktorName=dialog.result
        self.__w.SetzeTyp(faktorName, 'Bestand')
    
    def onFaktorFluss(self):
        '''weist einem Faktor die Eigenschaft Fluss zu sein zu.'''
        ## solange die Darstellung nicht geaendert wird, muss die Aktion
        ## nicht ueber den Controller gehen
        dialog=AuswahlDialog(self,
                             'Faktor waehlen',
                             self.__w.GibAlleFaktorNamen(),
                             True)
        if dialog.result==None: return # bei cancel None
        faktorName=dialog.result
        ## Ausgangsfaktor von waehlen
        vonDialog=AuswahlDialog(
            self,u'Ausgangsfaktor wählen',
            ['Quelle']+self.__w.GibAlleFaktorNamen(),
            True)
        if vonDialog.result==None: return # bei cancel None
        von=vonDialog.result
        ## Zielfaktor auf waehlen
        ziele=['Senke']+self.__w.GibAlleFaktorNamen()
        if  von in ziele: ziele.remove(von) # Kein Fluss in sich selbst
        aufDialog=AuswahlDialog(self,
                                u'Zielfaktor wählen',
                                ziele,
                                True)
        if aufDialog.result==None: return # bei cancel None
        auf=aufDialog.result
        self.__w.SetzeTyp(faktorName, 'Fluss',von,auf)
    
    def onFaktorParameter(self):
        '''weist einem Faktor die Eigenschaft Parameter zu sein zu.'''
        ## solange die Darstellung nicht geaendert wird, muss die Aktion
        ## nicht ueber den Controller gehen
        dialog=AuswahlDialog(self,
                             'Faktor waehlen',
                             self.__w.GibAlleFaktorNamen(),
                             True)
        if dialog.result==None: return # bei cancel None
        faktorName=dialog.result
        self.__w.SetzeTyp(faktorName, 'Parameter')

    def onFaktorZwischengroesse(self):
        '''weist einem Faktor die Eigenschaft Parameter zu sein zu.'''
        ## solange die Darstellung nicht geaendert wird, muss die Aktion
        ## nicht ueber den Controller gehen
        dialog=AuswahlDialog(self,
                             'Faktor waehlen',
                             self.__w.GibAlleFaktorNamen(),
                             True)
        if dialog.result==None: return # bei cancel None
        faktorName=dialog.result
        self.__w.SetzeTyp(faktorName, 'Zwischengroesse')
        
    def onWirkungBearbeiten(self):
        '''die hier einzig moegliche Bearbeitung ist das Aendern von
        verstaerkend oder daempfend'''
        wirkungen=Wirkung.GibAlleWirkungen()
        wirkungenVonAuf=[]
        for wirkung in wirkungen:
            wirkungenVonAuf.append(wirkung.ZeigeWirkung())
        dialog=AuswahlDialog(self,
                                'Wirkung wählen',
                                wirkungenVonAuf,
                                True)
        if dialog.result==None: return # bei cancel None
        vonAuf=dialog.result
        for i in range(len(wirkungen)):
            if wirkungenVonAuf[i]==vonAuf:
                wirkungen[i].AendereArt()
                tag=wirkungen[i].GibVon()+'->'+wirkungen[i].GibAuf()
                CanvasElement.ObjektZuTag(tag).AendereArt()
                self.__geaendert = True
                return
        
    #### ----------- Loeschen-Menu ----------------------------------------
    def onLoescheFaktor(self):
        dialog=AuswahlDialog(self,
                                'Faktor wählen',
                                self.__w.GibAlleFaktorNamen(),
                                True)
        if dialog.result==None: return # bei cancel None
        faktorName=dialog.result
        self.__w.LoescheFaktor(faktorName)
        self.__cont.LoescheFaktorUndWirkungen(self.zeichenflaeche,faktorName)
        self.__geaendert = True
        
    def onLoescheWirkung(self):
        vonFaktorNamen=[]
        for wirkung in Wirkung.GibAlleWirkungen():
            if not(wirkung.GibVon() in vonFaktorNamen):
                vonFaktorNamen.append(wirkung.GibVon())
        vonDialog=AuswahlDialog(self,
                                'Ausgangsfaktor wählen',
                                vonFaktorNamen,
                                True)
        if vonDialog.result==None: return # bei cancel None
        von=vonDialog.result
        aufFaktorNamen=[]
        for wirkung in Wirkung.GibAlleWirkungen():
            if not(wirkung.GibAuf() in aufFaktorNamen):
                if von==wirkung.GibVon():
                   aufFaktorNamen.append(wirkung.GibAuf())
        aufDialog=AuswahlDialog(self,
                                'Zielfaktor wählen',
                                aufFaktorNamen,
                                True)
        if aufDialog.result==None: return # bei cancel None
        auf=aufDialog.result
        wirkung=Wirkung.HatWirkung(von,auf)
        if not(wirkung): return # Wirkung gibt es nicht
        self.__w.LoescheWirkung(von,auf)
        self.__cont.LoescheWirkungElement(self.zeichenflaeche,von,auf)
        self.__geaendert = True
    
    #### ---------- Darstellungs-Menue -------------------------------------
    def onFenster(self):
        dialog=N_EingabenDialog(self,
                                u'Fenstergröße ändern',
                                [u'neue Breite (>=240)\nStandard 800',
                                 u'neue Fensterhöhe (>=240)\nStandard 800'],
                                [str(self.breite),str(self.hoehe)])
        if dialog.result==None: return # cancel ausgewaehlt
        try:
            breite=int(self.string_in_Zahl(dialog.result[0]))
            hoehe=int(self.string_in_Zahl(dialog.result[1]))
        except ValueError as e:
            self.Meldung(self, 'Fehler: '+str(e))
            return # falsche Werte
        self.breite = max(240,breite)
        self.hoehe = max(240,hoehe)
        self.zeichenflaeche.grid_forget()
        self.zeichenflaeche = Canvas(self.modellFrame,
                                     width=self.breite,
                                     height=self.hoehe,
                                     bg='white',
                                     scrollregion=(0, 0, 1800, 1200))
        self.zeichenflaeche.grid(row=0,column=0)
        self.scrollX.grid(row=1,column=0, sticky=W+E)
        self.scrollY.grid(row=0,column=1, sticky=N+S)

    def string_in_Zahl(self, ZahlString):
        '''Die Funktion bestimmt zu einem Zahlstring den Zahlenwert
        und gibt im Misserfolgsfall 0.0 zurueck.'''
        try:
            wert=float(ZahlString)
        except Exception:
            wert=0.0
        return wert

    def onModelldatenZeigen(self):
        self.__w.SichereModellDaten()
        zeilen=self.__w.ZeigeModellDaten()
        dialog=TextAusgabeDialog(self,
                                 'Modelldaten',
                                 self.__modellVariable.get(),
                                 zeilen)
        
    #### Mausereignisse  ----------------------------------------------
    def links_Mausklick(self, event):
        self.__click_position = (event.x,event.y)
        zf = self.zeichenflaeche
        ID = zf.find_closest(event.x, event.y)
        if ID==(): return # keine Elemente vorhanden
        if CanvasElement.IstFigurID(ID[0]): # keine Klicks auf Texte
            self.__click_ID = ID[0]
            tag=zf.gettags(self.__click_ID)[0]
            element=CanvasElement.ObjektZuTag(tag)
            if element.__class__==WirkungElement:
                element.AendereArt()
                von,pfeil,auf=tag.partition('->')
                wi=Wirkung.HatWirkung(von,auf)
                wi.AendereArt()
                self.__geaendert = True
            else:
                self.__ausgewaehlt=tag

    def rechts_Mausklick(self, event):
        '''ermoeglicht das Bearbeiten von Faktoren und Wirkungen'''
        zf = self.zeichenflaeche
        ID = zf.find_closest(event.x, event.y)
        if ID==(): return # keine Elemente vorhanden
        if CanvasElement.IstFigurID(ID[0]): # keine Klicks auf Texte
            tag=zf.gettags(ID[0])[0]
            element=CanvasElement.ObjektZuTag(tag)
            if element.__class__==WirkungElement:
                self.__cont.AendereBogenOrientierung(zf,tag)
                self.__geaendert = True
            else:
                self.onFaktorBearbeiten(tag)
            
            
    def links_loslassen(self, event):
        zf = self.zeichenflaeche
        if self.__click_ID!=None:
            ## erster Abschnitt fuer das Zeichnen einer Wirkung mit der Maus
            ID = zf.find_closest(event.x, event.y)
            if ID==(): return # keine Elemente vorhanden
            if self.__neueWirkung:
                if CanvasElement.IstFigurID(ID[0]): # keine Klicks auf Texte
                    #self.__release_IDs = (self.__click_ID, ID[0])
                    von=zf.gettags(self.__click_ID)[0]
                    auf=zf.gettags(ID[0])[0]
                    wi=self.__w.NeueWirkung(von, auf)
                    if wi!=None:
                        we=WirkungElement(zf, von, auf)
                        self.__cont.NeueWirkung(we.GibTag(),wi,we)
                        self.__geaendert = True
            else:  # verwendet das tag des ausgewaehlten Objektes
                faktor=Faktor.GibFaktorZuNamen(self.__ausgewaehlt)
                self.__geaendert = True
                self.__geaendert = True
                if faktor!=None:
                    wirkungsTags=[self.__ausgewaehlt]
                    for w in Wirkung.GibAlleWirkungen():
                        von,auf,art=w.GibVon(),w.GibAuf(),w.GibArt()
                        if (von in wirkungsTags)or(auf in wirkungsTags):
                            CanvasElement.AktualisiereWirkung(zf,von,auf,art)
        self.__ausgewaehlt = None
        self.__click_ID = None
        self.__neueWirkung = False

    def bewegt(self, event):
        if self.__neueWirkung: return
        if self.__click_ID != None:
            umX, umY = event.x - self.__click_position[0], event.y - self.__click_position[1]
            tags=self.zeichenflaeche.gettags(self.__click_ID)
            for tag in tags:
                if CanvasElement.IstFigurTag(tag): # Mehrfachzugriffe vermeiden
                    self.zeichenflaeche.move(tag, umX, umY)
            self.__click_position = (event.x,event.y)
    
    #### ------------------------------------------------------------------------
    def FaktorWaehlenDialog(self):
        faktorNamen=self.__w.GibAlleFaktorNamen()
        dialog=AuswahlDialog(self,u'Faktor wählen',faktorNamen, True)
        return dialog.result # bei cancel None
        
    def Meldung(self, text):
        MeldungDialog('Info', text)

    def ZeigeHilfeText1(self):
        TextAusgabeDialog(
            self,
            'Hilfe zum Export',
            'Export',
            ["Wirkungen auf Bestaende sind in Modelldiagrammen unzulaessig,\n"+
             "daher beim Festlegen eines Faktors als Bestand,\n"+
             " - einen Aenderungsfaktor einfuegen,\n"+
             " - diese Wirkungen darauf umlenken und\n"+
             " - eine Wirkung vom Aenderungsfaktor auf den Besatnd einbauen.\n\n"])
    def ZeigeHilfeText2(self):
        TextAusgabeDialog(
            self,
            u'Hilfe zur Modelldefinitionsdatei',
            u'Modelldefinition',
            [u"Muster: [('Faktor', 'x', (164.0, 337.0)), ..., ('Wirkung', 'x', 'a', '-'), ... ]"])
    def ZeigeHilfeText4(self):
        TextAusgabeDialog(
            self,
            u'Hilfe zu Kurztasten',
            u'Kurztasten',
            [u'Strg-o \t Datei öffnen',
             u'Strg-s \t Datei speichern',
             u'Strg-p \t Bild  speichern',
             u'Alt-n \t Neues Modell',
             u'Alt-f \t Neuer Faktor',
             u'Alt-w \t Neue  Wirkung',
             u'Alt-m \t Neue Wirkung mit Maus',
             u'Alt-z \t Modelldaten anzeigen'])
    def ZeigeHilfeText5(self):
        TextAusgabeDialog(
            self,
            u'Hilfe zum Ändern der Modellfenstergröße',
            u'Ändern der Modellfenstergröße',
            [u'Sichern und Beenden Sie das Projekt.',
            u'Ändern Sie dann in der Programmdatei',
            u'   zeichenflaechenBreite=600 in der siebtletzten Zeile bzw',
            u'   zeichenflaechenHoehe=600 in der sechstletzten Zeile.',
            u'Dann laden Sie das Projekt neu.'])
        
    #### Beispiele zeichnen
    def zeichne_Objekte(self):
        if not self.__cont.IstLeer():
            MeldungDialog(self,u'Achtung !','Fenster ist nicht leer!\nFür Beispiel Programm neu starten!')
            return
        zf = self.zeichenflaeche

        self.__modellVariable.set('Feder')

        f=self.__w.NeuerFaktor('x')
        fe=FaktorElement(zf, 'x', pos=(300,250))
        self.__cont.NeuerFaktor('x',f,fe)
        
        f1=self.__w.NeuerFaktor('a')
        fe1=FaktorElement(zf, 'a', pos=(120,150))
        self.__cont.NeuerFaktor('a',f1,fe1)

        wi=self.__w.NeueWirkung('x', 'a', '-')
        we=WirkungElement(zf, 'x', 'a', '-')
        self.__cont.NeueWirkung(we.GibTag(),wi,we)

        f2=self.__w.NeuerFaktor('v')
        fe2=FaktorElement(zf, 'v', pos=(150,300))
        self.__cont.NeuerFaktor('v',f2,fe2)
        
        wi2=self.__w.NeueWirkung('a', 'v')
        we2=WirkungElement(zf, 'a', 'v', '+')
        self.__cont.NeueWirkung(we2.GibTag(),wi2,we2)

        wi3=self.__w.NeueWirkung('v', 'x')
        we3=WirkungElement(zf, 'v', 'x', '+')
        self.__cont.NeueWirkung(we3.GibTag(),wi3,we3)

        f3=self.__w.NeuerFaktor('m')
        fe3=FaktorElement(zf, 'm', (180,50))
        self.__cont.NeuerFaktor('m',f3,fe3)

        wi4=self.__w.NeueWirkung('m', 'a', '-')
        we4=WirkungElement(zf, 'm', 'a', '-')
        self.__cont.NeueWirkung(we4.GibTag(),wi4,we4)

        f4=self.__w.NeuerFaktor('D')
        fe4=FaktorElement(zf, 'D', (80,50))
        self.__cont.NeuerFaktor('D',f4,fe4)

        wi5=self.__w.NeueWirkung('D', 'a')
        we5=WirkungElement(zf, 'D', 'a', '+')
        self.__cont.NeueWirkung(we5.GibTag(),wi5,we5)

#### als Hauptfenster starten
if __name__=='__main__':
    root = Tk()
    fenster = ExportApplication(master=root,
                                          zeichenflaechenBreite=800,
                                          zeichenflaechenHoehe=800)
    fenster.master.title('Wirkungsdiagramm in Modell')
    fenster.mainloop()
    try:
        root.destroy()
    except Exception: pass
